/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.fontbox.cff;

public abstract class CIDKeyedFDSelect {
	
	protected CFFFontROS owner = null;
	
	/**
	 * Constructor.
	 * @param _owner the owner of the FDSelect data.
	 */
	public CIDKeyedFDSelect(CFFFontROS _owner) {
		this.owner = _owner;
	}
	/**
	 * Returns the Font DICT index for the given glyph identifier
	 *  
	 * @param glyph
	 * @return -1 if the glyph isn't define, otherwise the FD index value
	 */
	public abstract int getFd(int glyph);
}
